/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.data.database.repository

import net.moonleay.bedge.data.database.entry.UserData
import net.moonleay.bedge.data.database.tables.UserTable
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.insert
import org.jetbrains.exposed.sql.select
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.transactions.transaction
import org.jetbrains.exposed.sql.update

object UserRepository {

    fun getAllUsers(): List<UserData> {
        val list: MutableList<UserData> = mutableListOf()
        transaction {
            UserTable.selectAll().forEach { user ->
                list.add(UserData(
                    user[UserTable.id],
                    user[UserTable.userid],
                    user[UserTable.accountCreationDate],
                    user[UserTable.currentStreak],
                    user[UserTable.longestStreak],
                    user[UserTable.wantsToBeReminded],
                    user[UserTable.coins],
                    user[UserTable.coinsCollected],
                    user[UserTable.xp],
                    user[UserTable.level],
                    user[UserTable.numberOfFails],
                    user[UserTable.targetChannelId],
                    user[UserTable.customReminderMessage],
                    user[UserTable.preferredSleepTime],
                    user[UserTable.customWakeupMessage],
                    user[UserTable.nextWakeup],
                    user[UserTable.nextWakeupCron],
                    user[UserTable.lastWakeup],
                    user[UserTable.isAwake],
                    user[UserTable.timezone]
                ))
            }
        }
        return list
    }


    fun getUserByID(id: ULong): UserData? = getUserByID(id.toLong())

    fun getUserByID(id: Long): UserData? {
        var user: UserData? = null
        transaction {
            UserTable.select(UserTable.userid eq id).forEach {
                user = UserData(
                    it[UserTable.id],
                    it[UserTable.userid],
                    it[UserTable.accountCreationDate],
                    it[UserTable.currentStreak],
                    it[UserTable.longestStreak],
                    it[UserTable.wantsToBeReminded],
                    it[UserTable.coins],
                    it[UserTable.coinsCollected],
                    it[UserTable.xp],
                    it[UserTable.level],
                    it[UserTable.numberOfFails],
                    it[UserTable.targetChannelId],
                    it[UserTable.customReminderMessage],
                    it[UserTable.preferredSleepTime],
                    it[UserTable.customWakeupMessage],
                    it[UserTable.nextWakeup],
                    it[UserTable.nextWakeupCron],
                    it[UserTable.lastWakeup],
                    it[UserTable.isAwake],
                    it[UserTable.timezone]
                )
            }
        }
        return user
    }

    fun doesUserExist(id: ULong) = transaction {
        UserTable.select { UserTable.userid eq id.toLong() }.count() > 0
    }

    fun write(data: UserData) = transaction {
        UserTable.insert {
            it[UserTable.userid] = data.userid
            it[UserTable.accountCreationDate] = data.accountCreationDate
            it[UserTable.currentStreak] = data.currentStreak
            it[UserTable.longestStreak] = data.longestStreak
            it[UserTable.wantsToBeReminded] = data.wantsToBeReminded
            it[UserTable.coins] = data.coins
            it[UserTable.coinsCollected] = data.coinsCollected
            it[UserTable.xp] = data.xp
            it[UserTable.level] = data.level
            it[UserTable.numberOfFails] = data.numberOfFails
            it[UserTable.targetChannelId] = data.targetChannelId
            it[UserTable.customReminderMessage] = data.customReminderMessage
            it[UserTable.preferredSleepTime] = data.preferredSleepTime
            it[UserTable.customWakeupMessage] = data.customWakeupMessage
            it[UserTable.nextWakeup] = data.nextWakeup
            it[UserTable.nextWakeupCron] = data.nextWakeupCron
            it[UserTable.lastWakeup] = data.lastWakeup
            it[UserTable.isAwake] = data.isAwake
            it[UserTable.timezone] = data.timezone
        } get UserTable.id
    }

    fun update(data: UserData) = transaction {
        UserTable.update({ UserTable.userid eq data.userid}) {
            it[UserTable.currentStreak] = data.currentStreak
            it[UserTable.longestStreak] = data.longestStreak
            it[UserTable.wantsToBeReminded] = data.wantsToBeReminded
            it[UserTable.coins] = data.coins
            it[UserTable.coinsCollected] = data.coinsCollected
            it[UserTable.xp] = data.xp
            it[UserTable.level] = data.level
            it[UserTable.numberOfFails] = data.numberOfFails
            it[UserTable.targetChannelId] = data.targetChannelId
            it[UserTable.customReminderMessage] = data.customReminderMessage
            it[UserTable.preferredSleepTime] = data.preferredSleepTime
            it[UserTable.customWakeupMessage] = data.customWakeupMessage
            it[UserTable.nextWakeup] = data.nextWakeup
            it[UserTable.nextWakeupCron] = data.nextWakeupCron
            it[UserTable.lastWakeup] = data.lastWakeup
            it[UserTable.isAwake] = data.isAwake
            it[UserTable.timezone] = data.timezone
        }
    }
}
