/*
 * Bedge
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.bedge.extensions

import com.kotlindiscord.kord.extensions.commands.Arguments
import com.kotlindiscord.kord.extensions.commands.converters.impl.optionalUser
import com.kotlindiscord.kord.extensions.extensions.Extension
import com.kotlindiscord.kord.extensions.extensions.publicSlashCommand
import com.kotlindiscord.kord.extensions.types.respond
import dev.kord.core.entity.User
import dev.kord.rest.builder.message.EmbedBuilder
import net.moonleay.bedge.data.database.entry.UserData
import net.moonleay.bedge.data.database.repository.UserRepository
import net.moonleay.bedge.util.EmbedColor
import net.moonleay.bedge.util.MessageUtil
import net.moonleay.bedge.util.TimeUtil

class ProfileExtension : Extension() {

    override val name = "profile"
    override val allowApplicationCommandInDMs: Boolean
        get() = false


    override suspend fun setup() {
        publicSlashCommand(::ProfileArguments) {
            name = "profile"
            description = "Checkout (your / a) profile"
            this.action {
                val user = this.user.asUser()
                if (this.arguments.target == null || this.arguments.target!!.id == this.user.id) {
                    // User wants to see own profile
                    val target = this.user.asUser()
                    if(!UserRepository.doesUserExist(target.id.value)){
                        this.respond {
                            this.embeds.add(
                                MessageUtil.getEmbed(
                                    EmbedColor.ERROR,
                                    "User not found!",
                                    "You did not use the bot yet, so there is no information about you.",
                                    target.username,
                                )
                            )
                        }
                        return@action
                    }
                    val td = UserRepository.getUserByID(target.id.value)!!
                    this.respond {
                        this.embeds.add(
                            getProfileMsg(td, target, target)
                        )
                    }
                    return@action
                }

                // User wants to see other profile
                val target = this.arguments.target!!
                if(!UserRepository.doesUserExist(target.id.value)){
                    this.respond {
                        this.embeds.add(
                            MessageUtil.getEmbed(
                                EmbedColor.ERROR,
                                "User not found!",
                                "The user did not use the bot yet, so there is no information about them.",
                                target.username,
                            )
                        )
                    }
                    return@action
                }
                val td = UserRepository.getUserByID(target.id.value)!!
                this.respond {
                    this.embeds.add(
                        getProfileMsg(td, target, user)
                    )
                }
            }
        }
    }

    private fun getProfileMsg(target: UserData, targetUser: User, requester: User): EmbedBuilder = MessageUtil.getEmbedWithImage(
        EmbedColor.INFO,
        "Profile of ${targetUser.username}",
        "Account created: ${TimeUtil.convertFromUnixStamp(target.accountCreationDate)}\n" +
                "Current streak: ${target.currentStreak} days\n" +
                "Longest streak: ${target.longestStreak} days\n" +
                "Number of fails: ${target.numberOfFails}x\n" +
                "Coins: ${target.coins}\n" +
                "Preferred sleep time: ${target.preferredSleepTime} hrs\n" +
                "Timezone: ${target.timezone}\n",
        requester.username,
        targetUser.avatar?.cdnUrl!!.toUrl())

    inner class ProfileArguments : Arguments() {
        val target by optionalUser {
            this.name = "target"
            this.description = "The target user"
        }
    }
}
